//@author: vux
//@help: template for texture fx
//@tags: texture
//@credits: 

Texture2D texture2d : PREVIOUS;

float timeIn ;

SamplerState linearSampler : IMMUTABLE
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

struct psInput
{
	float4 p : SV_Position;
	float2 uv : TEXCOORD0;
};

float hash( float n )
{
    return frac(sin(n)*43758.5453);
}
float noise( in float3 x )
{
    float3 p = floor(x);
    float3 f = frac(x);

    f = f*f*(3.0-2.0*f);
    float n = p.x + p.y*57.0 + 113.0*p.z;
    return lerp(lerp(lerp( hash(n+  0.0), hash(n+  1.0),f.x),
                   lerp( hash(n+ 57.0), hash(n+ 58.0),f.x),f.y),
               lerp(lerp( hash(n+113.0), hash(n+114.0),f.x),
                   lerp( hash(n+170.0), hash(n+171.0),f.x),f.y),f.z);
}

//x3
float3 noise3( in float3 x)
{
	return float3( noise(x+float3(123.456,.567,.37)),
				noise(x+float3(.11,47.43,19.17)),
				noise(x) );
}

//http://dept-info.labri.fr/~schlick/DOC/gem2.ps.gz
float bias(float x, float b) {
	return  x/((1./b-2.)*(1.-x)+1.);
}

float gain(float x, float g) {
	float t = (1./g-2.)*(1.-(2.*x));	
	return x<0.5 ? (x/(t+1.)) : (t-x)/(t-1.);
}


float3x3 rotation(float angle, float3 axis)
{
    float s = sin(-angle);
    float c = cos(-angle);
    float oc = 1.0 - c;
	float3 sa = axis * s;
	float3 oca = axis * oc;
    return float3x3(
		oca.x * axis + float3(c, -sa.z,	sa.y),
		oca.y * axis + float3( sa.z, c,	-sa.x),		
		oca.z * axis + float3(-sa.y, sa.x,	c));	
}

float3 fbm(float3 x, float H, float L, int oc)
{
	float3 v = float3(0, 0, 0);
	float f = 1.;
	for (int i=0; i<10; i++)
	{
		if (i >= oc) break;
		float w = pow(f,-H);
		v += noise3(x)*w;
		x *= L;
		f *= L;
	}
	return v;
}

float3 smf(float3 x, float H, float L, int oc, float off)
{
	float3 v = float3(1, 1, 1);
	float f = 1.;
	for (int i=0; i<10; i++)
	{
		if (i >= oc) break;
		v *= off + f*(noise3(x)*2.-1.);
		f *= H;
		x *= L;
	}
	return v;	
}


float4 PS(psInput input) : SV_Target
{

	float2 uv =2.0 * (input.uv) - 1;
	float time = timeIn* 1.276;
	float slow = time*0.002;
	uv *= 1. + .5*slow*sin(slow*10.);
	float ts = time*0.009;
	float change = gain(frac(ts),0.0008)+floor(ts);	//flick to a different view 
						
	float3 p = float3(uv*.2,slow+change);					//coordinate + slight change over time
	
	float3 axis = 4. * fbm(p, 0.5, 2., 8);				//random fbm axis of rotation
	
	float3 colorVec = 0.5 * 5. * fbm(p*0.3,0.5,2.,7);		//random base color
	p += colorVec;
	
//	float mag = 4e5;	//published, rather garish?
	float mag = 0.75e5; //still clips a bit
//	mag = mag * (1.+sin(2.*3.1415927*ts)*0.75);
	float3 colorMod = mag * smf(p,0.7,2.,8,.2);			//multifractal saturation
	colorVec += colorMod;
	
	colorVec = mul(rotation(3.*length(axis)+slow*10.,normalize(axis)),colorVec);
	
	colorVec *= 0.05;
			
//	colorVec = colorVec / (1. + length(colorVec));	//tone it all down a bit
	
	colorVec = pow(colorVec, float3(1./2.2, 1./2.2, 1./2.2));		//gamma
	float4 color = float4(colorVec,1.0);
	return color;
	
	
	
}

technique10 Process
{
	pass P0
	{
		SetPixelShader(CompileShader(ps_4_0,PS()));
	}
}



